pft = {

    'leafn': {
        'dim': 'pft',
        'ind': 2,
        'long_name': 'leaf N'
    },
    
    'leafn_storage': {
        'dim': 'pft',
        'ind': 3,
        'long_name': 'leaf N storage'
    },

    'leafn_xfer': {
        'dim': 'pft',
        'ind': 4,
        'long_name': 'leaf N transfer'
    },

    'frootn': {
        'dim': 'pft',
        'ind': 5,
        'long_name': 'fine root N'
    },

    'frootn_storage': {
        'dim': 'pft',
        'ind': 6,
        'long_name': 'fine root N storage'
    },

    'frootn_xfer': {
        'dim': 'pft',
        'ind': 7,
        'long_name': 'fine root N transfer'
    },

    'livestemn': {
        'dim': 'pft',
        'ind': 8,
        'long_name': 'live root N'
    },

    'livestemn_storage': {
        'dim': 'pft',
        'ind': 9,
        'long_name': 'live root N storage'
    },

    'livestemn_xfer': {
        'dim': 'pft',
        'ind': 10,
        'long_name': 'live root N transfer'
    },

    'deadstemn': {
        'dim': 'pft',
        'ind': 11,
        'long_name': 'dead stem N'
    },

    'deadstemn_storage': {
        'dim': 'pft',
        'ind': 12,
        'long_name': 'dead stem N storage'
    },

    'deadstemn_xfer': {
        'dim': 'pft',
        'ind': 13,
        'long_name': 'dead stem N transfer'
    },

    'livecrootn': {
        'dim': 'pft',
        'ind': 14,
        'long_name': 'live root N'
    },

    'livecrootn_storage': {
        'dim': 'pft',
        'ind': 15,
        'long_name': 'live root N storage'
    },

    'livecrootn_xfer': {
        'dim': 'pft',
        'ind': 16,
        'long_name': 'live root N transfer'
    },

    'deadcrootn': {
        'dim': 'pft',
        'ind': 17,
        'long_name': 'dead root N'
    },

    'deadcrootn_storage': {
        'dim': 'pft',
        'ind': 18,
        'long_name': 'dead root N storage'
    },

    'deadcrootn_xfer': {
        'dim': 'pft',
        'ind': 19,
        'long_name': 'dead root N transfer'
    },

    'npool': {
        'dim': 'pft',
        'ind': 20,
        'long_name': 'temporary plant N pool'
    },

    'leafc': {
        'dim': 'pft',
        'ind': 25,
        'long_name': 'leaf carbon'
    },

    'leafc_storage': {
        'dim': 'pft',
        'ind': 26,
        'long_name': 'leafc storage'
    },

    'leafc_xfer': {
        'dim': 'pft',
        'ind': 27,
        'long_name': 'leafc transfer'
    },

    'frootc': {
        'dim': 'pft',
        'ind': 29,
        'long_name': 'fine root C'    
    },

    'frootc_storage': {
        'dim': 'pft',
        'ind': 30,
        'long_name': ''    
    },

    'frootc_xfer': {
        'dim': 'pft',
        'ind': 31,
        'long_name': ''    
    },

    'livestemc': {
        'dim': 'pft',
        'ind': 32,
        'long_name': ''    
    },

    'livestemc_storage': {
        'dim': 'pft',
        'ind': 33,
        'long_name': ''    
    },

    'livestemc_xfer': {
        'dim': 'pft',
        'ind': 34,
        'long_name': ''    
    },

    'deadstemc': {
        'dim': 'pft',
        'ind': 35,
        'long_name': ''    
    },

    'deadstemc_storage': {
        'dim': 'pft',
        'ind': 36,
        'long_name': ''    
    },

    'deadstemc_xfer': {
        'dim': 'pft',
        'ind': 37,
        'long_name': ''    
    },

    'livecrootc': {
        'dim': 'pft',
        'ind': 38,
        'long_name': ''    
    },

    'livecrootc_storage': {
        'dim': 'pft',
        'ind': 39,
        'long_name': ''    
    },

    'livecrootc_xfer': {
        'dim': 'pft',
        'ind': 40,
        'long_name': ''    
    },

    'deadcrootc': {
        'dim': 'pft',
        'ind': 41,
        'long_name': ''    
    },

    'deadcrootc_storage': {
        'dim': 'pft',
        'ind': 42,
        'long_name': ''    
    },

    'deadcrootc_xfer': {
        'dim': 'pft',
        'ind': 43,
        'long_name': ''    
    },

    'gresp_storage': {
        'dim': 'pft',
        'ind': 44,
        'long_name': 'growth respiration storage'    
    },

    'gresp_xfer': {
        'dim': 'pft',
        'ind': 45,
        'long_name': ''    
    },

    'cpool': {
        'dim': 'pft',
        'ind': 46,
        'long_name': 'temporary photosynthate C pool'    
    },

    'totvegc': {
        'dim': 'pft',
        'ind': 48,
        'long_name': 'total vegetation carbon, excluding cpool'
    },

    'availc': {
        'dim': 'pft',
        'ind': 49,
        'long_name': 'C flux available for allocation'
    },

    'excess_cflux': {
        'dim': 'pft',
        'ind': 51,
        'long_name': 'C flux not allocated due to downregulation'
    }

}



soil = {

    'soil1n_vr': {
        'dim': ['column', 'levsoi'],
        'ind': 52,
        'long_name': 'SOIL1 (fast pool) N (vertically resolved)'
    },

    'soil2n_vr': {
        'dim': ['column', 'levsoi'],
        'ind': 53,
        'long_name': 'SOIL2 (medium pool) N (vertically resolved)'
    },

    'soil3n_vr': {
        'dim': ['column', 'levsoi'],
        'ind': 54,
        'long_name': 'SOIL3 (medium pool) N (vertically resolved)'
    },

    'soil1c_vr': {
        'dim': ['column', 'levsoi'],
        'ind': 55,
        'long_name': 'SOIL1 (fast pool) C (vertically resolved)'
    },

    'soil2c_vr': {
        'dim': ['column', 'levsoi'],
        'ind': 56,
        'long_name': 'SOIL2 (medium pool) C (vertically resolved)'
    },

    'soil3c_vr': {
        'dim': ['column', 'levsoi'],
        'ind': 57,
        'long_name': 'SOIL3 (medium pool) C (vertically resolved)'
    },
}