
EU3 = {
    'name': 'EUCORDEX3km',
    'version': (1, 0, 0),
    'path_file': '/p/scratch/cjibg31/jibg3105/CESMDataRoot/InputData/share/domains/',
    'name_file': 'EU3_CORDEX_gridfile_reducedim.nc',
    'type_file': 'netcdf',
    'name_latitude': 'XLAT_M',
    'name_longitude': 'XLONG_M',
    'name_landmask': 'LANDMASK',
    'lon_extents': [351.1, 57],
    'lat_extents': [27, 65.7],
}


GLEAM = {
    'name': 'GLEAM',
    'version': (3, 8, 1),
    'path_file': '/p/scratch/cjibg31/jibg3105/data/GLEAM/3.8a/yearly/',
    'name_file': '2000.nc',
    'type_file': 'netcdf',
    'name_latitude': 'lat',
    'name_longitude': 'lon',
    'lon_extents': [351.1, 57],
    'lat_extents': [27, 65.7],
}


ERA5L_EUCORDEX = {
    'name': 'ERA5-Land-EUCORDEX',
    'version': (1, 0, 0),
    'path_file': '/p/scratch/cjibg31/jibg3105/data/ERA5L/all/',
    'name_file': '2000.nc',
    'type_file': 'netcdf',
    'name_latitude': 'latitude',
    'name_longitude': 'longitude',
    'lon_extents': [351.1, 57],
    'lat_extents': [27, 65.7],
}

GLDAS_EUROCORDEX = {
    'name': 'GLDAS_EUROCORDEX',
    'version': (1, 0, 0),
    'path_file': '/p/scratch/cjibg31/jibg3105/data/GLDAS/NOAH025_M/',
    'name_file': '2000.nc',
    'type_file': 'netcdf',
    'name_latitude': 'lat',
    'name_longitude': 'lon',
    'lon_extents': [351.1, 57],
    'lat_extents': [27, 65.7],
}